/******************************************************************************
 * Copyright (C) 2006 Tetsuya Kimata <kimata@acapulco.dyndns.org>
 *
 * All rights reserved.
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any
 * damages arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any
 * purpose, including commercial applications, and to alter it and
 * redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software. If you use this
 *    software in a product, an acknowledgment in the product
 *    documentation would be appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must
 *    not be misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source
 *    distribution.
 *
 * $Id: BBSConfig.h 2063 2006-11-24 14:09:34Z svn $
 *****************************************************************************/

#ifndef BBS_CONFIG_H
#define BBS_CONFIG_H

#include "Environment.h"

#include <cstdlib>

#include "apr_shm.h"

#include "Macro.h"
#include "Uncopyable.h"

class PostFlowController;
class BBSThreadList;
class BBSThreadManager;
class BBSTemplate;

using namespace std;

/**
 * @brief 掲示板の設定を保持するクラス．
 */
class BBSConfig: public Uncopyable
{
public:
    typedef struct Item {
        const char *param;
        const char *desc;
    } item_t;
    enum template_type_t {
        INDEX_VIEW,
        COMMENT_VIEW,
        THREAD_VIEW,
        POST_THREAD_VIEW,
    };

    static const item_t BASE_URL;
    static const item_t DATA_DIRECTORY;
    static const item_t FILE_DIRECTORY;
    static const item_t TEMP_DIRECTORY;
    static const item_t INDEX_VIEW_TEMPLATE;
    static const item_t COMMENT_VIEW_TEMPLATE;
    static const item_t THREAD_VIEW_TEMPLATE;
    static const item_t POST_THREAD_VIEW_TEMPLATE;

    BBSConfig(apr_pool_t *pool);

    void init();
    void child_init();
    void finalize();

    void set_template(template_type_t type, const char *tmpl_file_path);

    BBSThreadList *get_thread_list() const
    {
        return thread_list_;
    };
    BBSThreadManager *get_thread_manager() const
    {
        return thread_manager_;
    };
    PostFlowController *get_flow_controller() const
    {
        return flow_controller_;
    };
    BBSTemplate *get_template(template_type_t type)
    {
        if (UNLIKELY(is_debug_mode)) {
            update_template(type);
        }

        return tmpls_[type];
    };

    const char *to_string(apr_pool_t *pool);

    // 面倒なので public
    const char *base_url;
    const char *data_dir_path;
    const char *file_dir_path;
    const char *temp_dir_path;
    bool is_debug_mode;

private:
    void check();
    void check_template();
    void load_template();
    void update_template(template_type_t type);

    apr_pool_t *pool_;

    apr_shm_t *list_shm_;
    apr_shm_t *manager_shm_;
    apr_shm_t *flow_controller_shm_;

    BBSThreadList *thread_list_;
    BBSThreadManager *thread_manager_;

    PostFlowController *flow_controller_;

    BBSTemplate *tmpls_[4];
};

#endif

// Local Variables:
// mode: c++
// coding: utf-8-dos
// End:
